#import "BWScaledImageView.h"

#import "BWLog.h"

static const char *cvsid = "$Id: BWScaledImageView.m,v 1.1.1.1 2002/03/25 01:36:51 markd Exp $";
static BWLogModule *logger;

@implementation BWScaledImageView

+ (void) initialize
{
    [super initialize];

    logger = [BWLogModule registerModule: @"BWScaledImageView"
			  withCVSId: cvsid];
} // initialize


- (id) initWithFrame: (NSRect) frame
{
    BWLog (logger, BWTrace, @"init with frame: %@", NSStringFromRect(frame));

    if (self = [super initWithFrame: frame]) {
	scaling = NSScaleNone;
    }

    return (self);

} // initWithFrame


- (NSRect) centerRect: (NSRect) inner  in:  (NSRect) outer
{
    NSRect result;

    float iwidth = inner.size.width;
    float iheight = inner.size.height;
    float owidth = outer.size.width;
    float oheight = outer.size.height;

    float x, y;

    x = outer.origin.x + (owidth - iwidth) / 2;
    y = outer.origin.y + (oheight - iheight) / 2;

    result = NSMakeRect (x, y, iwidth, iheight);
    
    return (result);

} // centerRect


typedef enum RectAxis {
    RectAxisHorizontal,
    RectAxisVertical
} RectAxis;


- (NSRect) fitRect: (NSRect) inner  into: (NSRect) outer  dimension: (RectAxis) dimension
{
    NSRect result;
    float proportion;

    if (dimension == RectAxisHorizontal) {
	proportion = inner.size.height / inner.size.width;
	result.size.width = outer.size.width;
	result.size.height = result.size.width * proportion;
	result.origin = inner.origin;

    } else if (dimension == RectAxisVertical) {
	proportion = inner.size.width / inner.size.height;
	result.size.height = outer.size.height;
	result.size.width = result.size.height * proportion;
	result.origin = inner.origin;

    } else {
	NSLog (@"fitRect:  bad dimension");
	result = NSZeroRect;
    }

    return (result);

} // fitRect with dimension


- (NSRect) fitRect: (NSRect) inner  into: (NSRect) outer
{
    NSRect result;
    float innerProportion;

    innerProportion = inner.size.width / inner.size.height;

    result = inner;
    if (innerProportion > 1) {
	// wider
	result = [self fitRect: inner  into: outer  dimension: RectAxisHorizontal];
    } else {
	// taller
	result = [self fitRect: inner  into: outer  dimension: RectAxisVertical];
    }


    if (result.size.width > outer.size.width) {
	result = [self fitRect: inner  into: outer  dimension: RectAxisHorizontal];
    }
    if (result.size.height > outer.size.height) {
	result = [self fitRect: inner  into: outer  dimension: RectAxisVertical];
    }
    if (result.size.width > outer.size.width) {
	result = [self fitRect: inner  into: outer  dimension: RectAxisHorizontal];
    }

    result = [self centerRect: result in: outer];

    return (result);

} // fitRect


- (void) drawRect: (NSRect) rect
{
    NSSize size;
    NSRect fromRect;
    NSRect toRect;
    rect = [self bounds];
    [[NSColor whiteColor] set];
    [NSBezierPath fillRect: rect];

    size = [image size];
    fromRect = NSMakeRect (0, 0, size.width, size.height);

    
    if (scaling == NSScaleProportionally) {
	toRect = [self fitRect: fromRect  into: rect];
    } else if (scaling == NSScaleNone) {
	toRect = [self centerRect: fromRect in: rect];
    } else if (scaling == NSScaleToFit) {
	toRect = rect;
    }

    [image drawInRect: toRect
	   fromRect: fromRect
	   operation: NSCompositeCopy
	   fraction: 1.0];

    [[NSColor blackColor] set];
    [NSBezierPath strokeRect: rect];

} // drawRect


- (NSImage *) image
{
    return (image);
} // image


- (void) setImage: (NSImage *) img
{
    BWLog (logger, BWTrace, @"setImage: %@", img);

    [img retain];
    [image release];
    image = img;
    [self setNeedsDisplay: YES];

} // setImage


- (void) setImageScaling: (NSImageScaling) newScaling
{
    BWLog (logger, BWTrace, @"setImageScaling: %d", newScaling);
    scaling = newScaling;
    [self setNeedsDisplay: YES];

} // setImageScaling


- (NSImageScaling) imageScaling
{
    return (scaling);
} // imageScaling


@end // BWScaledImageView


